==> tinyca-0.4.3.tar.gz: TinyCA/lib/REQ.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: REQ.pm,v 1.2 2002/09/27 19:51:04 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package REQ;

sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};

   bless($self, $class);
}

# create new request and key
sub create_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};

   my($action, $reqname, $reqfile, $keyfile, $parsed, $opts, $name);

   $opts->{'bits'} = 2048;

   $parsed = $main->{'CERT'}->parse_cert($main, 'CA');
   
   defined($parsed) || $main->print_error("Can't read Certificate");

   # set defaults
   if(defined $parsed->{'C'}) {
      $opts->{'C'} = $parsed->{'C'};
   }
   if(defined $parsed->{'ST'}) {
      $opts->{'ST'} = $parsed->{'ST'};
   }
   if(defined $parsed->{'L'}) {
      $opts->{'L'} = $parsed->{'L'};
   }
   if(defined $parsed->{'O'}) {
      $opts->{'O'} = $parsed->{'O'};
   }
   if(defined $parsed->{'OU'}) {
      $opts->{'OU'} = $parsed->{'OU'}->[0];
   }

   $action = $main->show_req_dialog($opts);

   return if $action eq "Cancel";

   while((not defined($opts->{'passwd'}) ||
         ($opts->{'passwd'} eq '') ||
          not defined($opts->{'passwd2'})) ||
         ($opts->{'passwd2'} eq '') ||
         $opts->{'passwd'} ne $opts->{'passwd2'}) { 
      $main->print_warning("Passwords are required or don't match");
      $action = $main->show_req_dialog($opts);
      return if $action eq "Cancel";
   }

   $name = _gen_name($opts);

   # print "DEBUG: name: $name\n";

   $reqname = MIME::Base64::encode($name, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$reqname.".pem";
   $keyfile = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$reqname.".pem";

   if(-s $reqfile || -s $keyfile) {
      $action = $main->show_req_overwrite_warning();
      return if($action eq 'Cancel');
   }
         
   $main->{'OpenSSL'}->newkey(
         'bits'    => $opts->{'bits'},
         'outfile' => $keyfile,
         'pass'    => $opts->{'passwd'}
         );

   if (not -s $keyfile) { 
      unlink($keyfile);
      $main->print_warning("newkey() failed");
      return;
   }

   $main->{'OpenSSL'}->newreq(
         'config'   => $main->{'CA'}->{$ca}->{'cnf'},
         'outfile'  => $reqfile,
         'keyfile'  => $keyfile,
         'pass'     => $opts->{'passwd'},
         'dn'       => [ $opts->{'C'},
                         $opts->{'ST'},
                         $opts->{'L'},
                         $opts->{'O'},
                         $opts->{'OU'},
                         $opts->{'CN'},
                         $opts->{'EMAIL'},
                         '',
                         ''
                       ],
         );

   if (not -s $reqfile) { 
      unlink($keyfile);
      unlink($reqfile);
      $main->print_warning("newreq() failed");
      return;
   }

   $main->create_mframe();

   return;
}

sub del_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'reqtab'}->{'list'}->info('selection');

   my($reqname, $req, $reqfile, $action);

   if(not defined $ind) {
      $main->print_warning("no request selected");
      return;
   }

   $reqname = $self->{'reqlist'}->[$ind];

   $req = MIME::Base64::encode($reqname, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$req.".pem";

   if(not -s $reqfile) {
      $main->print_warning("Request not found");
      return;
   }

   $action = $main->show_req_del_confirm();

   return if($action eq 'Cancel');

   unlink($reqfile);

   $main->create_mframe();

   return;
}

sub read_reqlist {
   my $self   = shift;
   my $main   = shift;
   my $ca     = $main->{'CA'}->{'actca'};
   my $reqdir = $main->{'CA'}->{$ca}->{'dir'}."/req";

   my ($f, $modt);

   my $reqlist = [];

   $modt = (stat($reqdir))[9];

   if(defined($self->{'lastread'}) &&
      $self->{'lastread'} >= $modt) {  
      return(0);
   }

   opendir(DIR, $reqdir) || do {
      $main->print_warning("can't open reqdir");
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;
      $f =~ s/\.pem//;
      push(@{$reqlist}, MIME::Base64::decode($f));
   }
   @{$reqlist} = sort(@{$reqlist});
   closedir(DIR);

   $self->{'reqlist'} = $reqlist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub sign_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'reqtab'}->{'list'}->info('selection');

   my($reqname, $req, $reqfile, $serial, $certout, $certfile,
      $passwd, $action, $certtype, $days, $time, $parsed);

   $time = time();
   
   if(not defined $ind) {
      $main->print_warning("No Request selected");
      return;
   }

   $reqname = $self->{'reqlist'}->[$ind];

   $req = MIME::Base64::encode($reqname, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$req.".pem";

   if(not -s $reqfile) {
      $main->print_warning("Request not found");
      return;
   }

   if(-s $main->{'CA'}->{$ca}->{'dir'}."/certs/".$req.".pem") {
      $action = $main->show_cert_overwrite_confirm();
      return if($action ne "Overwrite");
   }

   $action = $main->show_req_sign_dialog(
         \$passwd,
         \$days,
         \$certtype
         );

   return if($action eq 'Cancel');

   $parsed = $main->{'CERT'}->parse_cert($main, 'CA');

   defined($parsed) || $main->print_error("Can't read CA certificate");

   if(($time + ($days * 86400)) > $parsed->{'EXPDATE'}) {
      $action = $main->show_req_date_warning;
      return if($action eq 'Cancel');
   }

   $serial = $main->{'CA'}->{$ca}->{'dir'}."/serial";
   open(IN, "<$serial") || do {
      $main->print_warning("can't read serial");
      return;
   };
   $serial = <IN>;
   chomp($serial);
   close IN;

   $main->{'OpenSSL'}->signreq(
         'config'  => $main->{'CA'}->{$ca}->{'cnf'},
         'reqfile' => $reqfile,
         'days'    => $days,
         'pass'    => $passwd,
         'caname'  => $certtype."_ca"
         );

   $certout  = $main->{'CA'}->{$ca}->{'dir'}."/newcerts/".$serial.".pem";
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$req.".pem";

   if (not -s $certout) {
         $main->print_warning("signreq() failed");
         return;
   }
   open(IN, "<$certout") || do {
      $main->print_warning("can't open certout");
      return;
   };
   open(OUT, ">$certfile") || do {
      $main->print_warning("can't open certfile");
      return;
   };
   print OUT while(<IN>);
   close IN; close OUT;

   $action = $main->show_req_del_question();

   if($action eq "Yes") {
      unlink($reqfile);
   }

   undef($parsed);
   
   # force reread of certlist
   $main->{'CERT'}->read_certlist($main, 1);

   $main->create_mframe();
     
   return;
}

sub import_req {
   my $self = shift;
   my $main = shift;

   my($reqname, $reqfile, $action, $infile, $parsed, $req, $ca,
      $out, $in, $format, $file, $tmpfile, $name);

   $ca = $main->{'CA'}->{'actca'};

   $action = $main->show_req_import_dialog( \$infile,); 

   return if $action eq "Cancel";
   return if(not defined($infile));
   return if(not -s $infile);

   open(IN, "<$infile") || do {
      $main->print_warning("Can't read Request");
      return;
   };

   $in .= $_ while(<IN>);

   if($in =~ /-BEGIN[\s\w]+CERTIFICATE REQUEST-/i) {
      $format = "PEM";
      $file = $infile;
   } else {
      $format = "DER";
   }

   if($format eq "DER") {
      $in = $main->{'OpenSSL'}->convdata(
            'cmd'     => 'req',
            'main'    => $main,
            'data'    => $in,
            'inform'  => 'DER',
            'outform' => 'PEM'
            );

      $tmpfile = _mktmp($main->{'OpenSSL'}->{'tmp'}."/import");
   
      open(TMP, ">$tmpfile") || do {
         $main->print_warning("Can't create tmpfile");
         return;
      };
      print TMP $in;
      close(TMP);
      $file = $tmpfile;
   }

   $parsed = $main->{'OpenSSL'}->parsereq($main, $file);
   
   if(not defined($parsed)) {
      unlink($tmpfile);
      $main->print_warning("Can't parse Request");
      return;
   }
   
   $action = $main->show_req_import_verification($parsed);

   return if $action eq 'Cancel';

   $name = _gen_name($parsed);
   
   $reqname = MIME::Base64::encode($name, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$reqname.".pem";

   open(OUT, ">$reqfile") || do {
      unlink($tmpfile);
      $main->print_warning("Can't open outfile");
      return;
   };
   print OUT $in;
   close OUT;

   $main->create_mframe();

   return;
}

sub parse_req {
   my $self = shift;
   my $main = shift;
   my $name = shift;
   
   my ($parsed, $ca, $reqfile, $req);

   $ca = $main->{'CA'}->{'actca'};

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$name.".pem";

   $parsed = $main->{'OpenSSL'}->parsereq( $main, $reqfile);

   return($parsed);
}


sub _mktmp {
   my $base = shift;
   my @rand = ();
   my $ret  = '';

   do {
      for(my $i = 0; $i < 8; $i++) { 
         push(@rand, int(rand 26)+65);
      }
      my $end = pack("C8", @rand);
      $ret = $base.$end; 
   } while (-e $ret);

   return($ret);
}

sub _gen_name {
   my $opts = shift;
   my $name = '';

   foreach (qw(CN EMAIL OU O L ST C)) {
      if((not defined($opts->{$_})) || ($opts->{$_} eq '')) {
         $opts->{$_} = ".";
      }
      if($opts->{$_} ne '.' && not ref($opts->{$_})) {
         $name .= $opts->{$_};
      } elsif (ref($opts->{$_})) {
         $name .= $opts->{$_}->[0];
      } else {
         $name .= " ";
      }
      $name .= ":" if($_ ne 'C');
   }

   return($name);
}

1

# 
# $Log: REQ.pm,v $
# Revision 1.2  2002/09/27 19:51:04  sm
# Fixed typo in _gen_name()
#
#

==> tinyca-0.4.5.tar.gz: TinyCA/lib/REQ.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: REQ.pm,v 1.6 2003/06/19 21:46:43 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package REQ;

sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};

   bless($self, $class);
}

# create new request and key
sub create_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};

   my($action, $reqname, $reqfile, $keyfile, $parsed, $opts, $name);

   $opts->{'bits'} = 2048;

   $parsed = $main->{'CERT'}->parse_cert($main, 'CA');
   
   defined($parsed) || $main->print_error("Can't read Certificate");

   # set defaults
   if(defined $parsed->{'C'}) {
      $opts->{'C'} = $parsed->{'C'};
   }
   if(defined $parsed->{'ST'}) {
      $opts->{'ST'} = $parsed->{'ST'};
   }
   if(defined $parsed->{'L'}) {
      $opts->{'L'} = $parsed->{'L'};
   }
   if(defined $parsed->{'O'}) {
      $opts->{'O'} = $parsed->{'O'};
   }
   if(defined $parsed->{'OU'}) {
      $opts->{'OU'} = $parsed->{'OU'}->[0];
   }

   $action = $main->show_req_dialog($opts);

   return if $action eq "Cancel";

   while((not defined($opts->{'passwd'}) ||
         ($opts->{'passwd'} eq '') ||
          not defined($opts->{'passwd2'})) ||
         ($opts->{'passwd2'} eq '') ||
         $opts->{'passwd'} ne $opts->{'passwd2'}) { 
      $main->print_warning("Passwords are required or don't match");
      $action = $main->show_req_dialog($opts);
      return if $action eq "Cancel";
   }

   $name = _gen_name($opts);

   # print "DEBUG: name: $name\n";

   $reqname = MIME::Base64::encode($name, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$reqname.".pem";
   $keyfile = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$reqname.".pem";

   if(-s $reqfile || -s $keyfile) {
      $action = $main->show_req_overwrite_warning();
      return if($action eq 'Cancel');
   }
         
   $main->{'OpenSSL'}->newkey(
         'bits'    => $opts->{'bits'},
         'outfile' => $keyfile,
         'pass'    => $opts->{'passwd'}
         );

   if (not -s $keyfile) { 
      unlink($keyfile);
      $main->print_warning("newkey() failed");
      return;
   }

   $main->{'OpenSSL'}->newreq(
         'config'   => $main->{'CA'}->{$ca}->{'cnf'},
         'outfile'  => $reqfile,
         'keyfile'  => $keyfile,
         'pass'     => $opts->{'passwd'},
         'dn'       => [ $opts->{'C'},
                         $opts->{'ST'},
                         $opts->{'L'},
                         $opts->{'O'},
                         $opts->{'OU'},
                         $opts->{'CN'},
                         $opts->{'EMAIL'},
                         '',
                         ''
                       ],
         );

   if (not -s $reqfile) { 
      unlink($keyfile);
      unlink($reqfile);
      $main->print_warning("newreq() failed");
      return;
   }

   $main->create_mframe();

   return;
}

sub del_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'reqtab'}->{'list'}->info('selection');

   my($reqname, $req, $reqfile, $action);

   if(not defined $ind) {
      $main->print_warning("no request selected");
      return;
   }

   $reqname = $self->{'reqlist'}->[$ind];

   $req = MIME::Base64::encode($reqname, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$req.".pem";

   if(not -s $reqfile) {
      $main->print_warning("Request not found");
      return;
   }

   $action = $main->show_req_del_confirm();

   return if($action eq 'Cancel');

   unlink($reqfile);

   $main->create_mframe();

   return;
}

sub read_reqlist {
   my $self   = shift;
   my $main   = shift;
   my $ca     = $main->{'CA'}->{'actca'};
   my $reqdir = $main->{'CA'}->{$ca}->{'dir'}."/req";

   my ($f, $modt, $d);

   my $reqlist = [];

   $modt = (stat($reqdir))[9];

   if(defined($self->{'lastread'}) &&
      $self->{'lastread'} >= $modt) {  
      return(0);
   }

   opendir(DIR, $reqdir) || do {
      $main->print_warning("can't open reqdir");
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;
      $f =~ s/\.pem//;
      $d = MIME::Base64::decode($f);
      next if not defined($d);
      next if $d eq "";
      push(@{$reqlist}, $d);
   }
   @{$reqlist} = sort(@{$reqlist});
   closedir(DIR);

   $self->{'reqlist'} = $reqlist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub sign_req {
   my $self     = shift;
   my $main     = shift;
   my $certtype = shift;
   my $ca       = $main->{'CA'}->{'actca'};
   my $ind      = $main->{'reqtab'}->{'list'}->info('selection');

   my($reqname, $req, $reqfile, $serial, $certout, $certfile,
      $passwd, $action, $days, $time, $parsed);

   $time = time();
   
   if(not defined $ind) {
      $main->print_warning("No Request selected");
      return;
   }

   $reqname = $self->{'reqlist'}->[$ind];

   $req = MIME::Base64::encode($reqname, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$req.".pem";

   if(not -s $reqfile) {
      $main->print_warning("Request not found");
      return;
   }

   if(-s $main->{'CA'}->{$ca}->{'dir'}."/certs/".$req.".pem") {
      $action = $main->show_cert_overwrite_confirm();
      return if($action ne "Overwrite");
   }

   $days = $main->{'CONFIG'}->{$certtype."_ca"}->{'default_days'};

   $action = $main->show_req_sign_dialog(
         \$passwd,
         \$days
         );

   return if($action eq 'Cancel');

   $parsed = $main->{'CERT'}->parse_cert($main, 'CA');

   defined($parsed) || $main->print_error("Can't read CA certificate");

   if(($time + ($days * 86400)) > $parsed->{'EXPDATE'}) {
      $action = $main->show_req_date_warning;
      return if($action eq 'Cancel');
   }

   $serial = $main->{'CA'}->{$ca}->{'dir'}."/serial";
   open(IN, "<$serial") || do {
      $main->print_warning("can't read serial");
      return;
   };
   $serial = <IN>;
   chomp($serial);
   close IN;

   $main->{'OpenSSL'}->signreq(
         'config'  => $main->{'CA'}->{$ca}->{'cnf'},
         'reqfile' => $reqfile,
         'days'    => $days,
         'pass'    => $passwd,
         'caname'  => $certtype."_ca"
         );

   $certout  = $main->{'CA'}->{$ca}->{'dir'}."/newcerts/".$serial.".pem";
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$req.".pem";

   if (not -s $certout) {
         $main->print_warning("signreq() failed");
         return;
   }
   open(IN, "<$certout") || do {
      $main->print_warning("can't open certout");
      return;
   };
   open(OUT, ">$certfile") || do {
      $main->print_warning("can't open certfile");
      return;
   };
   print OUT while(<IN>);
   close IN; close OUT;

   $action = $main->show_req_del_question();

   if($action eq "Yes") {
      unlink($reqfile);
   }

   undef($parsed);
   
   # force reread of certlist
   $main->{'CERT'}->read_certlist($main, 1);

   $main->create_mframe();
     
   return;
}

sub import_req {
   my $self = shift;
   my $main = shift;

   my($reqname, $reqfile, $action, $infile, $parsed, $req, $ca,
      $out, $in, $format, $file, $tmpfile, $name);

   $ca = $main->{'CA'}->{'actca'};

   $action = $main->show_req_import_dialog( \$infile,); 

   return if $action eq "Cancel";
   return if(not defined($infile));
   return if(not -s $infile);

   open(IN, "<$infile") || do {
      $main->print_warning("Can't read Request");
      return;
   };

   $in .= $_ while(<IN>);

   if($in =~ /-BEGIN[\s\w]+CERTIFICATE REQUEST-/i) {
      $format = "PEM";
      $file = $infile;
   } else {
      $format = "DER";
   }

   if($format eq "DER") {
      $in = $main->{'OpenSSL'}->convdata(
            'cmd'     => 'req',
            'main'    => $main,
            'data'    => $in,
            'inform'  => 'DER',
            'outform' => 'PEM'
            );

      $tmpfile = _mktmp($main->{'OpenSSL'}->{'tmp'}."/import");
   
      open(TMP, ">$tmpfile") || do {
         $main->print_warning("Can't create tmpfile");
         return;
      };
      print TMP $in;
      close(TMP);
      $file = $tmpfile;
   }

   $parsed = $main->{'OpenSSL'}->parsereq($main, $file);
   
   if(not defined($parsed)) {
      unlink($tmpfile);
      $main->print_warning("Can't parse Request");
      return;
   }
   
   $action = $main->show_req_import_verification($parsed);

   return if $action eq 'Cancel';

   $name = _gen_name($parsed);
   
   $reqname = MIME::Base64::encode($name, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$reqname.".pem";

   open(OUT, ">$reqfile") || do {
      unlink($tmpfile);
      $main->print_warning("Can't open outfile");
      return;
   };
   print OUT $in;
   close OUT;

   $main->create_mframe();

   return;
}

sub parse_req {
   my $self = shift;
   my $main = shift;
   my $name = shift;
   
   my ($parsed, $ca, $reqfile, $req);

   $ca = $main->{'CA'}->{'actca'};

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$name.".pem";

   $parsed = $main->{'OpenSSL'}->parsereq( $main, $reqfile);

   return($parsed);
}


sub _mktmp {
   my $base = shift;
   my @rand = ();
   my $ret  = '';

   do {
      for(my $i = 0; $i < 8; $i++) { 
         push(@rand, int(rand 26)+65);
      }
      my $end = pack("C8", @rand);
      $ret = $base.$end; 
   } while (-e $ret);

   return($ret);
}

sub _gen_name {
   my $opts = shift;
   my $name = '';

   foreach (qw(CN EMAIL OU O L ST C)) {
      if((not defined($opts->{$_})) || ($opts->{$_} eq '')) {
         $opts->{$_} = ".";
      }
      if($opts->{$_} ne '.' && not ref($opts->{$_})) {
         $name .= $opts->{$_};
      } elsif (ref($opts->{$_})) {
         $name .= $opts->{$_}->[0];
      } else {
         $name .= " ";
      }
      $name .= ":" if($_ ne 'C');
   }

   return($name);
}

1

# 
# $Log: REQ.pm,v $
# Revision 1.6  2003/06/19 21:46:43  sm
# change button status dynamically
#
# Revision 1.4  2002/10/04 09:02:50  sm
# fixed typo
#
# Revision 1.3  2002/10/04 09:01:51  sm
# avoid empty lines in list if decoding of filenames failed
# set days to 365 default
#
# Revision 1.2  2002/09/27 19:51:04  sm
# Fixed typo in _gen_name()
#
#
